<?php

namespace DMG\Newsletters;

/**
 * Class Post_Type
 *
 * Register the new post type to support newsletter signups.
 *
 * @package DMG\Newsletters
 */
class Post_Type {

	/**
	 * Post type name.
	 * 
	 * @var string
	 */
	const NAME = 'dmg_newsletter';

	/**
	 * Post type capability.
	 * 
	 * @var string
	 */
	const CAPABILITY = 'manage_dmg_newsletters';

	/**
	 * Init.
	 * 
	 * @return void
	 */
	public static function init(): void {
		add_action( 'init', array( __CLASS__, 'register' ) );
		add_action( 'enqueue_block_editor_assets', array( __CLASS__, 'enqueue_block_editor_assets' ) );
		add_action( 'admin_init', array( __CLASS__, 'add_capability' ) );
		add_action( 'admin_menu', array( __CLASS__, 'remove_menu_page' ) );
	}

	/**
	 * Register the post type.
	 * 
	 * @return void
	 */
	public static function register(): void {
		register_post_type(
			self::NAME,
			array(
				'label'        => 'Newsletters',
				'labels'       => array(
					'singular_name' => 'Newsletter',
					'add_new'       => 'Add New Newsletter',
					'edit_item'     => 'Edit Newsletter',
				),
				'description'  => 'Post type for managing individual newsletter subscriptions.',
				'public'       => true,
				'menu_icon'    => 'dashicons-email-alt',
				'show_in_rest' => true,
				'has_archive'  => true,
				'rewrite'      => array(
					'slug' => 'newsletters',
				),
				'supports'     => array(
					'title',
					'editor',
					'excerpt',
					'thumbnail',
					'custom-fields',
					'page-attributes'
				),
				'taxonomies'   => array(
					'category',
					'post_tag'
				),
				'map_meta_cap' => true,
				'capabilities' => array(
					'read'                   => self::CAPABILITY,
					'create_posts'           => self::CAPABILITY,
					'edit_posts'             => self::CAPABILITY,
					'delete_posts'           => self::CAPABILITY,
					'publish_posts'          => self::CAPABILITY,
					'edit_published_posts'   => self::CAPABILITY,
					'delete_published_posts' => self::CAPABILITY,
					'edit_others_posts'      => self::CAPABILITY,
					'delete_others_posts'    => self::CAPABILITY,
					'read_private_posts'     => self::CAPABILITY,
					'edit_private_posts'     => self::CAPABILITY,
					'delete_private_posts'   => self::CAPABILITY,
				)
			)
		);
	}

	/**
	 * Enqueue block editor assets.
	 * 
	 * @return void
	 */
	public static function enqueue_block_editor_assets(): void {

		// Post type check.
		if ( self::NAME !== get_post_type() ) {
			return;
		}

		// Add button for converting to post
		$asset = require_once( dirname( __FILE__ ) . '/build/editor.asset.php' );

		wp_enqueue_script(
			'dmg-newsletters-editor',
			plugin_dir_url( __FILE__ ) . 'build/editor.js',
			$asset['dependencies'],
			$asset['version'],
			array(
				'in_footer' => true
			)
		);
	}

	/**
	 * Add the default capability to administrators.
	 * 
	 * @return void
	 */
	public static function add_capability() {
		$role = get_role( 'administrator' );
		$role->add_cap( self::CAPABILITY );
	}

	/**
	 * Remove the menu page for Newsletters if no capability present.
	 * 
	 * (the menu item is present by default due to the category and
	 * tag taxonomy relationships, and as such, the menu needs to be
	 * removed by targeting the first submenu item for user without
	 * the capability, which is the category list page)
	 * 
	 * @return void
	 */
	public static function remove_menu_page() {
		if ( ! self::current_user_can() ) {
			remove_menu_page(
				'edit-tags.php?taxonomy=category&amp;post_type=' . self::NAME
			);
		}
	}

	/**
	 * Helper to determine if the current user can manage newsletters.
	 * 
	 * @return bool If the current user has newsletter management capabilities.
	 */
	public static function current_user_can(): bool {
		return (bool) current_user_can( self::CAPABILITY );
	}

}

Post_Type::init();
