/**
 * WordPress dependencies.
 */
import {
	select,
	useSelect,
	useDispatch
} from '@wordpress/data';
import {
	compose,
	withInstanceId
} from '@wordpress/compose';
import {
	TextControl,
	TextareaControl,
	ToggleControl
} from '@wordpress/components';
import { PluginDocumentSettingPanel } from '@wordpress/edit-post';

/**
 * Types.
 */
import { NewsletterMeta } from '../types';

/**
 * JSX element to display the newsletter Settings meta panels.
 *
 * @returns {*} JSX
 */
const Panels = () => {

	// Get newsletter meta.
	const meta: NewsletterMeta = useSelect( ( select ) => {
		const { getEditedPostAttribute } = select('core/editor');

		return {
			sailthruList: getEditedPostAttribute('dmg_newsletter_sailthru_list'),
			frequency: getEditedPostAttribute('dmg_newsletter_frequency'),
			msnText: getEditedPostAttribute('dmg_newsletter_msn_text'),
			hasPostcode: getEditedPostAttribute('dmg_newsletter_has_postcode'),
			hasBirthDate: getEditedPostAttribute('dmg_newsletter_has_birth_date')
		};
	}, [] );

	const {
		sailthruList,
		frequency,
		msnText,
		hasPostcode,
		hasBirthDate
	} = meta;

	const { editPost } = useDispatch('core/editor');

	return (
		<>
			<PluginDocumentSettingPanel
				name="dmg-newsletter__settings"
				title="Settings"
				className="dmg-newsletter__settings"
			>
				<TextControl
					label="Sailthru List"
					value={ sailthruList }
					help="The Sailthru list ID to send subscriptions to."
					onChange={ (newSailthruList: string) => editPost( { dmg_newsletter_sailthru_list: newSailthruList } ) }
				/>
				<TextControl
					label="Frequency"
					value={ frequency }
					help="Freeform field to indicate how frequently the newsletter is sent."
					onChange={ (newFrequency: string) => editPost( { dmg_newsletter_frequency: newFrequency } ) }
				/>
				<TextareaControl
					label="MSN Text"
					value={ msnText }
					help="Text to use for MSN feed."
					onChange={ (newMSNText: string) => editPost( { dmg_newsletter_msn_text: newMSNText } ) }
				/>
			</PluginDocumentSettingPanel>
			<PluginDocumentSettingPanel
				name="dmg-newsletter__fields"
				title="Fields"
				className="dmg-newsletter__fields"
			>
				<p>Please select from the following optional fields to include in the signup form:</p>
				<ToggleControl
					label="Postcode"
					checked={ hasPostcode }
					onChange={ (checked: boolean) => editPost( { dmg_newsletter_has_postcode: checked } ) }
				/>
				<ToggleControl
					label="Date of Birth"
					checked={ hasBirthDate }
					onChange={ (checked: boolean) => editPost( { dmg_newsletter_has_birth_date: checked } ) }
				/>
				<p class="description"><small><em>Note: all newsletters have the email field by default.</em></small></p>
			</PluginDocumentSettingPanel>
		</>
	);
}

export default Panels;
